#import <UnitKit/UnitKit.h>
#import <Foundation/Foundation.h>
#import "GNUstep.h"

@interface PredicateTest: NSObject <UKTest>
{
	NSMutableDictionary *dict;
	NSPredicate *p;
}
@end

@implementation PredicateTest

- (void) testString
{
	p = [NSPredicate predicateWithFormat: @"%K == %@", @"Record1.Name", @"John"];
	UKTrue([p evaluateWithObject: dict]);
	p = [NSPredicate predicateWithFormat: @"%K MATCHES[c] %@", @"Record1.Name", @"john"];
	UKTrue([p evaluateWithObject: dict]);
	p = [NSPredicate predicateWithFormat: @"%K BEGINSWITH %@", @"Record1.Name", @"Jo"];
	UKTrue([p evaluateWithObject: dict]);
	p = [NSPredicate predicateWithFormat: @"(%K == %@) AND (%K == %@)", @"Record1.Name", @"John", @"Record2.Name", @"Mary"];
	UKTrue([p evaluateWithObject: dict]);
}

- (void) testNumber
{
	p = [NSPredicate predicateWithFormat: @"%K == %d", @"Record1.Age", 34];
	UKTrue([p evaluateWithObject: dict]);
	p = [NSPredicate predicateWithFormat: @"%K < %d", @"Record1.Age", 40];
	UKTrue([p evaluateWithObject: dict]);
	p = [NSPredicate predicateWithFormat: @"%K BETWEEN %@", @"Record1.Age", [NSArray arrayWithObjects: [NSNumber numberWithInt: 20], [NSNumber numberWithInt: 40], nil]];
	UKTrue([p evaluateWithObject: dict]);

	p = [NSPredicate predicateWithFormat: @"(%K == %d) OR (%K == %d)", @"Record1.Age", 34, @"Record2.Age", 34];
	UKTrue([p evaluateWithObject: dict]);
}

- (void) testAttregate
{
	p = [NSPredicate predicateWithFormat: @"%@ IN %K", @"Kid1", @"Record1.Children"];
	UKTrue([p evaluateWithObject: dict]);
#endif
#if 0
	p = [NSPredicate predicateWithFormat: @"Any %K == %@", @"Record2.Children", @"Girl1"];
	UKTrue([p evaluateWithObject: dict]);
}

- (void) testKVC
{
	UKStringsEqual(@"A Title", [dict valueForKey: @"title"]);
	UKStringsEqual(@"A Title", [dict valueForKeyPath: @"title"]);
	UKStringsEqual(@"John", [dict valueForKeyPath: @"Record1.Name"]);
	UKIntsEqual(30, [[dict valueForKeyPath: @"Record2.Age"] intValue]);
}

- (id) init
{
	self = [super init];
	dict = [[NSMutableDictionary alloc] init];
	[dict setObject: @"A Title" forKey: @"title"];
	NSDictionary *d = [NSDictionary dictionaryWithObjectsAndKeys:
			@"John", @"Name",
			[NSNumber numberWithInt: 34], @"Age",
			[NSArray arrayWithObjects: @"Kid1", @"Kid2", nil], @"Children",
			nil];
	[dict setObject: d forKey: @"Record1"];
	d = [NSDictionary dictionaryWithObjectsAndKeys:
			@"Mary", @"Name",
			[NSNumber numberWithInt: 30], @"Age",
			[NSArray arrayWithObjects: @"Kid1", @"Girl1", nil], @"Children",
			nil];
	[dict setObject: d forKey: @"Record2"];
	return self;
}

- (void) dealloc
{
	DESTROY(dict);
	[super dealloc];
}

@end
